# RISC 2008. Demo 2 - Lists etc.
#
#
# This is a GAP demonstration file. 
# To run a demonstration enter the following:
# gap> ReadLib("demo.g");
# gap> Demonstration("filename.g");
# You may need the full path in the last command.
# Then press <Enter> to go to the next step or
# press <q> to terminate the demonstration.
#
# -------------------------------------------------
#
# Try some things with lists, sets and records
;
l := [ 2, 3, 5, 7 ];
l[1];
l[3];
# l[9]; returns an error (which would break the demo)
IsBound( l[9] );
l[9] := 50;
l;
l[8] := [1,2];
l;
;
# some functions for lists
;
3 in l;
6 in l;
Length( l );
# Length tells us the index of the highest bound entry
;
Number( l );
# Number tells us how many bound entries there are
;
Position( l, 7 );
# (what would happen if 7 appeared twice in our list?)
;
Position( l, 10 );
List( l, x -> x^2 );
Append( l, [51,52] );
Add( l, 53 );
l;
Concatenation( l, [54,55] );
# difference to Append?
;
l;
m := [ 1 .. 100 ]; # a range
7 in m;
m[43];
Filtered( m, IsPrime );
Primes;
Collected( List( last, x -> x mod 7 ) );
First( [10000,10003..19999], x -> IsPrime(x) );
Position( Primes2, last );
Primes2[ Length( Primes2 ) ];
LogInt( last, 10 ); # ?logint
;
# Further list functions
;
r := [1,3..15]; # a range with a step other than +1
List( r, x -> CyclicGroup( x ) );
# this suggests a typical use for ranges in loops...
;
ForAll( r, IsPrimeInt );
ForAny( r, IsPrimeInt );
First( r, IsPrimeInt );
Number( r, IsPrimeInt );
Reversed( r );
# we could input such a range at the prompt
;
# Identical lists
;
l1 := [3,4,5];
l2 := l1;
# two variables may refer to the same list
;
l1[3] := 6;
l2; # changes to it are visible to either (unlike earlier for cyclotomics)
l2 := ShallowCopy( l1 ); # to make a different list with the same elements
l1[2] := 5;
l1;
l2;
MakeImmutable( l1 ); # can no longer change l1
l2 := ShallowCopy( l1 );
l2[2] := -1;
l1;
l2;
# See also StructuralCopy
;
# Sets
;
k := [ 5, 3, 2, 5, , , 100, -1 ];
IsSet( k ); # is it dense, mutable and duplicate free?
k1 := Set( k ); # Set is a synonym for SSortedList
AddSet( k1, 1 );
k1;
AddSet( k1, 5 );
k1;
Intersection( k1, [1,2,4] );
Union( k1, [1,2,2] ); # may use a list that is not a set as an argument
;
# Strings
;
String( Factorial( 15 ) );
IsList( last );
last2[10]; # why ask for last2, not last?
;
# Vectors and matrices
;
v := [1,2,1/2];
IsRowVector( v );
v * 1/2;
1/3 * v;
v + v;
v * v; # scalar product
m := [ ShallowCopy( v ), ShallowCopy( v ) + 1, [2,3,4] ];
m[3][1];
IsMatrix( m ); # dense list of row vectors of equal length
m^2;
Inverse( m );
v*m;
m*v;
Z(3)^0*m;
Display( last );
IdentityMat( 3 );
DiagonalMat( v );
;
# Records
;
me := rec( name := "john", surname := "mcdermott", 
phone := 3813, department := [ "maths", "comp sci" ] );
me.name;
IsBound( me.address );
RecNames( me );
me.address := rec( line1 := "st andrews", line2 := "scotland" );
me.address.line2;
Display( me );
Unbind( me.surname );
me;
