# RISC 2008. Demo 4 - Constructing objects
# Examples based on a web page by David Joyner
#
# This is a GAP demonstration file. 
# To run a demonstration enter the following:
# gap> ReadLib("demo.g");
# gap> Demonstration("<demo-name>.g");
# You may need the full path in the last command.
# Then press <Enter> to go to the next step or
# press <q> to terminate demonstration.
#
# -------------------------------------------------
#
# Package loading; data library access and other constructions
#
;
LoadPackage("polycyclic");
# already there
;
LoadPackage("browse");
# We see Browse had a pre-requisite, or suggested package, AtlasRep
;
;
# Accessing the data libraries
;
;
AllSmallGroups( 12, IsAbelian );
;
NrPrimitiveGroups( 10 );
;
OneTransitiveGroup( NrMovedPoints, [17], Size, [1..30] );
;
;
# Advertisement - user question from the GAP forum on Tuesday 8 July 2008:
;
# The function IsomorphismGroups(G,H) (and others like it) will return a
# "random" isomorphism between the groups G and H (if one exists) but is
# there a way to find all such isomorphisms?
;
G := Group( [ (1,2), (3,4) ] );
iso := IsomorphismGroups( G, SmallGroup(4,2) );
allisos := List( AutomorphismGroup(G), alpha -> alpha*iso );
;
# and there's another data library access in there...
;
# Basic group constructions
;
;
# Permutation groups
g7 := Group( (1,2), (1,2,3,4,5,6,7) );
Size( g7 );
s7 := SymmetricGroup( 7 );
Size( s7 );
a7 := AlternatingGroup( 7 );
IsSubgroup( g7, a7 );
;
# Dihedral groups
;
d12 := DihedralGroup( 12 );
Size( d12 );
gens := GeneratorsOfGroup( d12 );
gens[1]^2;
gens[2]^2;
gens[3]^6;
Subgroup( d12, [ gens[2], gens[1]*gens[3] ] );
Index( d12, last );
;
# Cyclic groups
;
R := ZmodnZ( 12 );
els := Elements( R );
6*els[3];
els[3]+els[4];
;
# or...
c := CyclicGroup( 12 );
a := Random( c );
gens := GeneratorsOfGroup( c );
# !!
;
gens[1]^2=gens[2];gens[2]^2=gens[3];
cp := CyclicGroup( IsPermGroup, 12 );
cm := CyclicGroup( IsMatrixGroup, 12 );
;
# Finitely presented groups
;
F := FreeGroup( 2 );
# f1 is not a variable you can call
;
G := F / [ F.1^2, F.2^3, F.1*F.2*F.1^-1*F.2^-1 ];
Size( G );
;
# Conjugacy classes example - find abelian subgroups of A5
;
G := AlternatingGroup( 5 );
classes := ConjugacyClassesSubgroups( G );
cl := classes[4];
len := Size( cl ); 
rep := Representative( cl );
ord := Size( rep );
IsAbelian( rep );
abel := Filtered( classes, cl -> IsAbelian( Representative( cl ) ) );
;
# What is abel?
;
# See also ?classical groups
;
# Enough about groups for now
;
;
# Polynomials - construction
;
pts := Z(7)^0*[1,2,3];
vals := Z(7)^0*[1,2,6];
poly := InterpolatedPolynomial( GF(7), pts, vals );
;
# or we can do
;
x := X( Rationals );
f := x + x^2 + 1;
Value( f, [x], [1] );
Factors( f );
;
# See the interface to Singular, the GAP package of the same name
;
# Also try RadiRoot. Ex - access the radiroot documentation at the prompt
;
# Another approach to finding the root of a polynomial
;
rootOfPolynomial := function( f, R )
 local F0, Ff, a;
 F0 := CoefficientsRing(R);
 Ff := FieldExtension(F0,f);
 a := RootOfDefiningPolynomial(Ff);
 return a;
end;
;
F := Rationals;
x := X( F, 1 );
f := x^2 + x + 1;
R := PolynomialRing( F, [x] );
a := rootOfPolynomial( f, R );
Value( f, [x], [a] );
;
# Do ?E and ?X
;
# Let's do some arithmetic over a polynomial ring
;
Z4 := ZmodnZ(4);
R := UnivariatePolynomialRing( Z4, 1 );
x := IndeterminatesOfPolynomialRing( R )[1];
I := TwoSidedIdealByGenerators( R, [x^8-x^0] );
gen := x^8-x^0;
QuotientRemainder( R, x^8, gen );
PowerMod( R, x, 15, gen );
;
