# RISC 2008. Demo 6 - Operations and Methods
#
#
# This is a GAP demonstration file. 
# To run a demonstration enter the following:
# gap> ReadLib("demo.g");
# gap> Demonstration("<demo-name>.g");
# You may need the full path in the last command.
# Then press <Enter> to go to the next step or
# press <q> to terminate demonstration.
#
# -------------------------------------------------
#
;
# Understanding which methods are being used
;
FamilyObj( ZmodnZ( 3 ) );
;
# See ?collections
;
KnownAttributesOfObject( ZmodnZ( 3 ) );
KnownPropertiesOfObject( ZmodnZ( 3 ) );
KnownTruePropertiesOfObject( ZmodnZ( 3 ) );
;
# Test some of these
;
IsEmpty( ZmodnZ(3) );
IsFiniteDimensional( ZmodnZ(3) );
;
# why?
;
# Note:
;
Integers mod 3 = GF(3); GF(3) = ZmodnZ(3);
;
TraceMethods( [ Size ] );
Size( SymmetricGroup( 4 ) );
Size( Group( (1,2), (3,4) ) );
Size( DihedralGroup( 4 ) );
;
TraceImmediateMethods( true );
Size( Group( (1,3) ) );
;
# Switch off tracing:
;
TraceImmediateMethods( false );
UntraceMethods( [Size] );
;
;
# A new operation/method
;
# The exponent of a finite group G is the least common multiple
# of the orders of the elements of G.
;
DeclareOperation( "MyExponent", [ IsGroup ] );
;
InstallMethod( MyExponent,
    "Abelian method based on generators",
    true,
    [ HasGeneratorsOfGroup and IsAbelian and IsGroup ],
    0,
    function( g )
    	      if Length( GeneratorsOfGroup( g ) ) = 0
	      then return 1;
	      else return Lcm( List( GeneratorsOfGroup( g ), Order ) );
	      fi;
    end );
;
InstallMethod( MyExponent, "Default method",
	       true, [ IsGroup ], 0,
	       function( g )
	       local e, x;
	       e := 1;
	       for x in g do e := Lcm( e, Order(x) ); od;
	       return e;
	       end );
;
# Test it
;
gps := [ SmallGroup( 7, 1 ), SmallGroup( 12, 2 ), SmallGroup( 9, 2 ) ];
TraceMethods( MyExponent );
List( gps, MyExponent );
;
# why?
;
HasIsAbelian( gps[2] );
IsAbelian( gps[2] );
MyExponent( gps[2] );
;
UntraceMethods( MyExponent );